@@ -463,6 +463,12 @@ cost to performance.""")
     t.textLine('canvas.rect(x, y, width, height) - x,y is lower left')
 
     c.roundRect(inch,6.25*inch,2*inch,0.6*inch,0.1*inch)
+    c.saveState()
+    c.setStrokeColor(red)
+    c.roundRect(inch*1.1,6.35*inch,1.8*inch,0.4*inch,[0.1*inch,0,0.09*inch,0])
+    c.setStrokeColor(green)
+    c.roundRect(inch*1.15,6.40*inch,1.7*inch,0.3*inch,[0,0.1*inch,0,0.09*inch])
+    c.restoreState()
     t.setTextOrigin(4*inch, 6.55*inch)
     t.textLine('canvas.roundRect(x,y,width,height,radius)')
 
@@ -1119,6 +1125,22 @@ class PdfgenTestCase(unittest.TestCase):
             canv.drawString(x+80+1,y+23,'Alpha=%.1f' % alpha)
             canv.rect(x+90,y+10,10,10,fill=1)
             canv.setFillOverprint(op)
+            canv.setBlendMode('Multiply')
+            canv.setBlendMode('Screen')
+            canv.setBlendMode('Overlay')
+            canv.setBlendMode('Darken')
+            canv.setBlendMode('Lighten')
+            canv.setBlendMode('ColorDodge')
+            canv.setBlendMode('ColorBurn')
+            canv.setBlendMode('HardLight')
+            canv.setBlendMode('SoftLight')
+            canv.setBlendMode('Difference')
+            canv.setBlendMode('Exclusion')
+            canv.setBlendMode('Hue')
+            canv.setBlendMode('Saturation')
+            canv.setBlendMode('Color')
+            canv.setBlendMode('Luminosity')
+            canv.setBlendMode('Normal')
             canv.drawString(x+120+1,y+3,'OP=%d' % int(op))
             canv.drawString(x+120+1,y+23,'Alpha=%.1f' % alpha)
             canv.rect(x+130,y+10,10,10,fill=1)
@@ -1225,6 +1247,30 @@ class PdfgenTestCase(unittest.TestCase):
         c.showPage()
         c.save()
 
+    def testSpooledTemporarySaves(self):
+        return
+        from tempfile import SpooledTemporaryFile
+        with SpooledTemporaryFile() as f:
+            c = canvas.Canvas(f)
+            c.drawString(100, 700, 'Hello World I write to %s' % repr(f))
+            c.showPage()
+            c.save()
+
+    def testBytesIOSaves(self):
+        from io import BytesIO
+        with BytesIO() as f:
+            c = canvas.Canvas(f)
+            c.drawString(100, 700, 'Hello World I write to %s' % repr(f))
+            c.showPage()
+            c.save()
+
+    def testBadFileName(self):
+        f = ['filename']
+        c = canvas.Canvas(f)
+        c.drawString(100, 700, 'Hello World I write to %s' % repr(f))
+        c.showPage()
+        self.assertRaises(TypeError,c.save)
+
 def trySomeColors(C,enforceColorSpace=None):
     from reportlab.lib.utils import getBytesIO
     out=getBytesIO()
